
/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include <usart.h>
#include "system\typedefs.h"

#include "system\usb\usb.h"

#include "io_cfg.h"             // I/O pin mapping
#include "user\user.h"






/** V A R I A B L E S ********************************************************/
#pragma udata
byte counter;
double k = 0.02328306;		// 100 Mhz = 0.02328306;  80 Mhz = 0.01862645;  66.667 Mhz = 0.01552212
DATA_PACKET dataPacket;





/** P R I V A T E  P R O T O T Y P E S ***************************************/

void BlinkUSBStatus(void);
void ServiceRequests(void);
BOOL UBit(unsigned char byteB, unsigned char bit0);
void UsartFunc(unsigned char dataByte);
void setCodeFreq(byte b1, byte b2, byte b3, byte b4);
void setFreq(long frequency);
byte ReadEEPROM(byte addr);






/** D E C L A R A T I O N S **************************************************/
#pragma code



void UserInit(void)
{
    // 0 - output
    TRISA &= 0b00101111; LATA &= 0x00;
    TRISB &= 0b00011111; LATB &= 0x00;
    TRISC &= 0b00111000; LATC &= 0x00;
    ADCON2=0b10111100;
    mLED_1 = 1;								//    DDS ( RC0  "1"  _-) 
    mLED_1 = 0;								//   RC0  "0"  -_
    

    setCodeFreq(ReadEEPROM(0x51), ReadEEPROM(0x61), ReadEEPROM(0x71), ReadEEPROM(0x81));
    //setFreq(40000);					
}





void ProcessIO(void)
{   
    BlinkUSBStatus();
    if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
    ServiceRequests();
}







void ServiceRequests(void)
{
    byte index;
    
    if(USBGenRead((byte*)&dataPacket,sizeof(dataPacket)))
    {
        counter = 0;
        if(dataPacket.CMD == PKTBYTE){		

		//  
		setCodeFreq(dataPacket.ID1, dataPacket.ID2, dataPacket.ID3, dataPacket.ID4);
		dataPacket._byte[0] = dataPacket.ID1;	// ok
		dataPacket._byte[1] = dataPacket.ID2;	// ok
		dataPacket._byte[2] = dataPacket.ID3;	// ok
		dataPacket._byte[3] = dataPacket.ID4;	// ok
             	counter=0x04;	// 4 

	}else if(dataPacket.CMD == READ_INFO){
		
		EEADR = dataPacket.ADDR;	//  
		EECON1bits.EEPGD = 0; //   
		EECON1bits.RD = 1; //     
		dataPacket._byte[0] = EEDATA;

		//EECON1 = 0b00000000;

		counter = 0x01;
		
	}else if(dataPacket.CMD == SAVE_INFO){
		
		EEADR = dataPacket.ADDR; //  
		EEDATA = dataPacket.VALUE;    // 
		EECON1 = 0b00000100;
		 
		EECON2 = 0x55;
		EECON2 = 0x0AA;
		EECON1bits.WR = 1;  //    
		 
		EECON1bits.WREN = 0;	//   
		dataPacket._byte[0] = EEDATA;

		counter = 0x01;
		
	}else if(dataPacket.CMD == RESET) Reset();	


        if(counter != 0){
            if(!mUSBGenTxIsBusy())
                USBGenWrite((byte*)&dataPacket,counter);
        }
    }

}



byte ReadEEPROM(byte addr){
	EEADR = addr;	//  
	EECON1bits.EEPGD = 0; //   
	EECON1bits.RD = 1; //     
	return EEDATA;
}



void BlinkUSBStatus(void)
{
    static word led_count=0;
    
    if(led_count == 0){
	  led_count = 30000U;
	  LATAbits.LATA4 = 0;	// BUSY 
    }
    if(led_count == 5000)LATAbits.LATA4 = 1;
    led_count--;
}



void setCodeFreq(byte b1, byte b2, byte b3, byte b4){
	UsartFunc(b1);			
	UsartFunc(b2);			
	UsartFunc(b3);			
	UsartFunc(b4);
	UsartFunc(0x00);		
	mLED_1 = 1;								//    DDS ( RC0  "1"  _-) 
	mLED_1 = 1;								//    DDS ( RC0  "1"  _-) 
	mLED_1 = 0;								//   RC0  "0"  -_
}



void setFreq(long frequency){
	frequency = frequency / k;
	UsartFunc(frequency % 256);			
	UsartFunc(frequency / 256);			
	UsartFunc(frequency / 65536);			
	UsartFunc(frequency / 16777216);
	UsartFunc(0x00);		
	mLED_1 = 1;								//    DDS ( RC0  "1"  _-) 
	mLED_1 = 1;								//    DDS ( RC0  "1"  _-) 
	mLED_1 = 0;								//   RC0  "0"  -_
}

void UsartFunc(unsigned char dataByte)					// dataByte -   
{	
	static byte i;
	for(i = 0; i < 8; i++){
		mLED_3 = UBit(dataByte, i);						//  RC2   i-    
		mLED_2 = 1;										//  CLOCK  "1" 		_-
		mLED_2 = 0;										//  CLOCK  "0"		-_

	}
}// end UsartFunc

BOOL UBit(unsigned char byteB, unsigned char bit0)
{
	if(((byteB >> bit0) & 1) > 0)
	{
		return 1;
	}else{
		return 0;
	}
}

